/*      Heteroskedastic Probit 
**
**      See R. Michael Alvarez and John Brehm  
**      "American Ambivalence Towards Abortion Policy: 
**      Development of a Heteroskedastic Probit Model of Competing Values" 
**      American Journal of Political Science 39(4): 1055-1082.       
**
*/
/*use maxlik;*/
library maxlik;

dataset = "abdata";

let dep= abany;       
let ind=black male cath reliten attend erameans era;
let het=abproct abconct abprocon abimp abinfo abfirm;
/*
stval = {2.547836, -.5118134, -.0792347, -.5205102, -.3910329, -1.038167, -.0178301, .3261382, -.139939, .1688777, -.0439967, .0051383, .3712365, -.3705249};
*/
stval = ones(14,1);

proc loglik(b,dta);                         @ log-likelihood function @
    local xb,x1,zb,z,y,llik,cdf;
    y=dta[.,1];                             @ break up dta @
    x1=ones(rows(dta),1)~dta[.,2:8];
    z =dta[.,9:14];
    xb=x1*b[1:8]; 
    zb=z*b[9:14];
    cdf = cdfn(xb./(exp(zb)));
    llik=y.*ln(cdf)+(1-y).*ln(1-cdf);
    retp( llik );
endp;

proc gradlk(b,dta);
    local xb,x1,zb,z,y,grad,cdf,pdf;
    y=dta[.,1];
    x1=ones(rows(dta),1)~dta[.,2:8];
    z = dta[.,9:14];
    xb=x1*b[1:8]; 
    zb=z*b[9:14];
    cdf = cdfn(xb./(exp(zb)));
    pdf = pdfn(xb./(exp(zb)));
    grad = y.*(pdf./cdf).*(x1./(exp(zb)))+(1-y).*(-pdf./(1-cdf)).*(x1./(exp(zb)))~y.*(pdf./cdf).*(-xb.*z./(exp(zb)))+(1-y).*(-pdf./(1-cdf)).*(-xb.*z./(exp(zb))); 
    retp(grad);
endp;


/* set maxlik parameters and output */
__title="Heteroskedastic Probit Model";
_mlparnm="const"|ind|het;                  @ parameter names @
vars=dep|ind|het;                          @ vars to read in @
t=dataset$+".out";
output file=^t on;

_max_GradProc=&gradlk;  
/*_max_GradCheckTol = 1e-3;*/
_maxCovPar=3;

{b,logl,g,vc,ret}=maxprt(maxlik(dataset,vars,&loglik,stval));
output off;
