/*      Scobit or Skewed Logit 
**
**      See Jonathan Nagler. 1994. "Scobit: An Alternative 
**      Estimator to Logit and Probit." American Journal of 
**      Political Science 38(1):230-255. 
**
*/
/*use maxlik;*/
library maxlik;
dataset="cps";                        @ name data set @
let dep=vote;                          @ name dependent variable @
let ind=educ educ2 age age2 south gov closing edclose ed2close;                        @ name independent variables @

/*stval=zeros(rows(ind)+1,1);            @ starting values set to zeros @
stval=stval+.025*/

stval = {-5.3290,.3516,.0654,.1822,-.0013,-.2817,.0003,-.0012,-.0055,.0002,.4150};

proc loglik(b,dta);                         @ log-likelihood function @
    local xb,x1,alphab,alpha,y,llik;
    y=dta[.,1];                             @ break up dta @
    x1=ones(rows(dta),1)~dta[.,2:cols(dta)];
    alpha = ones(rows(dta),1);
    xb=x1*b[1:cols(dta)]; 
    alphab=alpha*b[cols(dta)+1];
    llik=y.*(ln(1-(1+exp(xb)).^(-alphab)))+(1-y).*(ln((1+exp(xb)).^(-alphab)));
    retp( llik );
endp;

proc gradlk(b,dta);
    local xb,x1,alphab,alpha,y,burr,grad;
    y=dta[.,1];
    x1=ones(rows(dta),1)~dta[.,2:cols(dta)];
    alpha = ones(rows(dta),1);
    xb=x1*b[1:cols(dta)];
    alphab=alpha*b[cols(dta)+1];
    burr = (1+exp(xb)).^(-alphab);
    grad = ((y./(1-burr))-((1-y)./burr)).*(alphab).*exp(xb).*((1+exp(xb)).^(-(alphab+1))).*x1~((-y./(1-burr))+(1-y)./burr).*(-ln(1+exp(xb))).*burr; 
    retp(grad);
endp;


/* set maxlik parameters and output */
__title="Scobit Model";
_mlparnm="const"|ind;                  @ parameter names @
vars=dep|ind;                          @ vars to read in @
t=dataset$+".out";
output file=^t on;

_max_IterData[3]=="BOOT";
_max_GradProc=&gradlk;
_max_GradCheckTol = 1e-3;
_maxCovPar=3;

{b,logl,g,vc,ret}=maxprt(maxlik(dataset,vars,&loglik,stval));

print "alternate stderrs";
se = real(sqrt(diag(_max_HessCov)));
print se;
se = real(sqrt(diag(_max_XprodCov)));
print se;
{ b,logl,g,vc,ret} = maxprt(MAXBoot(dataset,vars,&loglik,b));
output off;
